<?php

/*

	Plugin Name: Syndicate Out
	Plugin URI: http://blogs.fubra.com/jonathon/category/syndicate-out/
	Version: 0.4
	Description: Syndicates posts made in any specified category to another WP blog using WordPress' built in XML-RPC functionality.
	Author: Jonathon Wardman
	Author URI: http://www.flutt.co.uk/

	TODO: Deal with post updates
	        - Posts are no longer re-posted. More needs to be done.
	      Deal with remote comments.

*/

#
#  syndicate_out.php
#
#  Created by Jonathon Wardman on 09-07-2009.
#  Copyright 2009, Fubra Limited. All rights reserved.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  You may obtain a copy of the License at:
#  http://www.gnu.org/licenses/gpl-3.0.txt
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.


	 // Nothing in this plugin works outside of the admin area, so don't bother
	 // loading it if we're not looking at the admin panel...
if ( is_admin() ) {

 /* Setup section. */

	 // Register functions...
	add_action( 'admin_menu', 'syndicate_out_menu' );
	add_action( 'admin_init', 'syndicate_out_register_settings' );
	add_action( 'wp_insert_post', 'syndicate_out_post' );
	add_action( 'pre_post_update', 'syndicate_out_edit', 5 );
	
    // Register the plugin activation function...
	register_activation_hook( __FILE__, 'syndicate_out_activate');

 /* Admin section. */
 
	 // Admin menu...
	function syndicate_out_menu() {

		add_submenu_page( 'options-general.php', 'Syndicate Out Settings', 'Syndication', 'manage_options', 'syndicate_out', 'syndicate_out_admin' );

	}

	 // Register valid admin options...
	function syndicate_out_register_settings() {

		register_setting( 'syndicate-out-options', 'so_options', 'syndicate_out_sanitize_options' );

	}

	 // Admin page...
	function syndicate_out_admin() {

		require_once( 'so-options.php' );

	}

 /* Post / action section. */

	 // Global variables relating to posts...
	$postIsEdit = false;

	 // Sanitize the all settings...
	function syndicate_out_sanitize_options( $options ) {
	
		switch ($options['syndicate_category']) {
			case 'syndication': case 'all': case 'none': break;
			default: $options['syndicate_category'] = 'none'; break;
		}
	
		foreach ( $options['servers'] AS $serverKey => $serverDetails ) {
			if ( '' == $serverDetails['server'] ) {
				unset( $options['servers'][$serverKey] );
			} else {
				$remoteServer = trim( $serverDetails['server'] );
				if ( 'http://' != substr( $remoteServer, 0, 7 ) ) {
		         $remoteServer = 'http://'.$remoteServer;
				}
				if ( '/' != substr( $remoteServer, -1 ) ) {
		         $remoteServer .= '/';
				}
				$options['servers'][$serverKey]['server'] = $remoteServer;
			}
		}
		
		return $options;
	
	}

	 // Carry out the syndication on post insert...
	function syndicate_out_post( $postId ) {

		$soOptions = get_option( 'so_options' );
		if ( in_category( $soOptions['category'], $postId ) ) {

			$postData = get_post( $postId );

			if ( 'publish' == $postData->post_status ) {
				if ( @include_once(  ABSPATH . WPINC . '/class-IXR.php' ) ) {

					$remotePost['title'] = $postData->post_title;
					$remotePost['description'] = $postData->post_content;
					if ( $postTags = syndicate_out_get_tags( $postId ) ) {
						$keywords = array();
						foreach ( $postTags AS $postTag ) {
							$keywords[] = $postTag->name;
						}
						$remotePost['mt_keywords'] = implode( ',', $keywords );
					}
					if ( 'none' != $soOptions['syndicate_category'] ) {
						if ( 'syndication' == $soOptions['syndicate_category'] ) {
							$remotePost['categories'] = array( get_cat_name( $soOptions['category'] ) );
						} else if ( 'all' == $soOptions['syndicate_category'] ) {
							$remotePost['categories'] = array();
							$categories = get_the_category( $postId );
							foreach ( $categories AS $postCategory ) {
                        $remotePost['categories'][] = $postCategory->cat_name;
							}
						}
					}

				}
			}
			
  		if ( false === $GLOBALS['postIsEdit'] ) {
  			$remotePostInformation = array();
  			foreach ( $soOptions['servers'] AS $serverKey => $serverDetails ) {
  				$xmlrpc = new IXR_Client( $serverDetails['server'].'xmlrpc.php' );
  				$xmlrpc->query( 'metaWeblog.newPost', 1, $serverDetails['username'], $serverDetails['password'], $remotePost, 1 );
  				$remotePostInformation[$serverKey] = $xmlrpc->getResponse();
  			}
  			add_post_meta( $postId, '_so_remote_posts', serialize( $remotePostInformation ) );
  		} else {
        $remotePostIds = unserialize( get_post_meta( $postId, '_so_remote_posts', true ) );
        foreach ( $remotePostIds AS $serverKey => $remotePostId ) {
          if ( isset( $soOptions['servers'][$serverKey] ) ) {
    				$xmlrpc = new IXR_Client( $soOptions['servers'][$serverKey]['server'].'xmlrpc.php' );
    				$xmlrpc->query( 'metaWeblog.editPost', $remotePostId, $soOptions['servers'][$serverKey]['username'], $soOptions['servers'][$serverKey]['password'], $remotePost, 1 );
          }
  			}
  		}

		}

	}

	 // Mark edited posts as edits (not new posts)...
	function syndicate_out_edit( $postId ) {

		$postData = get_post( $postId );
		if ( 'draft' != $postData->post_status ) {
			$GLOBALS['postIsEdit'] = true;
		}

	}

	 // Get a list of tags for this post...
	function syndicate_out_get_tags( $postId ) {

		$terms = get_object_term_cache( $postId, 'post_tag' );
		if ( false === $terms ) {
			$terms = wp_get_object_terms( $postId, 'post_tag' );
		}

		if ( empty( $terms ) ) {
			return false;
		}

		return $terms;

	}
	
 /* Maintenance section. */
 
	 // Activation function.  Updates the old settings storage format to the
	 // new settings storage format...
	function syndicate_out_activate() {

		if ( $oldSettingsCategory = get_option( 'so_category' ) ) {
			$oldSettingsServer = get_option( 'so_remote_server' );
			$oldSettingsUsername = get_option( 'so_remote_username' );
			$oldSettingsPassword = get_option( 'so_remote_password' );
			$newSettings = array( 'category' => $oldSettingsCategory,
			                      'syndicate_category' => 'none',
			                      'servers' => array( array( 'server' => $oldSettingsServer,
										                            'username' => $oldSettingsUsername,
										                            'password' => $oldSettingsPassword ) ) );
			update_option( 'so_options', $newSettings );
			delete_option( 'so_category' );
			delete_option( 'so_remote_server' );
			delete_option( 'so_remote_username' );
			delete_option( 'so_remote_password' );

		}

	}

}